@@ -1,4 +1,5 @@
 import logging
+import warnings
 
 import dotenv
 
@@ -12,4 +13,4 @@ for path in ["openai", "anthropic", "gemini", "deepseek", "openrouter"]:
     try:
         __import__(__package__ + "." + path)
     except Exception as e:
-        logging.warning("%s not supported, please install missing modules", path, exc_info=True)
\ No newline at end of file
+        pass
\ No newline at end of file
